/* yasep/core-js/txt_int.js
This file is distributed under terms of the Affero GPL v3 license or later, see http://yasep.org
created Sat Mar 5 23:33:09 EST 2005 by whygee@f-cpu.org as vsp_arch.js

just some text-to-int and int-to-text for the YASEP simulator

Wed Mar  9 18:41:30 EST 2005 : Object-Oriented re-shaping of the code in order to clean it as much as possible.
Wed Jun  8 20:38:45 EDT 2005 : adding memory
2 days later : moving memory to vsp_mem.js and the registers to vsp_reg.js

2005-06-15 : updating the message queue's code and fixed int2hex
2005-06-17 : message queue now looking much better and int2hex is re-fixed
2005-06-19 : colors added to the messages
2006-08-03 : added str2int() [and removed equivalent code from other places]
2006-08-06 : split from vsp_arch.js (the name is inadapted)
2006-08-09 : added support of h/H and negative numbers in str2int(),
  added the optional maxlength argument and the trailing 'h' to int2hex()
2006-08-10 : validation
2006-08-12 : added 'quiet' argument and binary format, + misc. bugfixes
2006-09-02 : moved to vspsim/JScore/ and added an overflow warning
2007-04-06 : added some features (opt_h and minlength) to int2hex()
2007-11-05 : rebranding to YASEP
2009-07-30 : some code syntax compaction
2011-08-08 : directory renamed to core-js
2012-02-12 : added toBin
2012-09-21 : str2int : changed the parameters : quiet=>add_message (there is no global function anymore)

dependency :
  yasep_messages.js, or define an empty function add_message() otherwise.

Warning : the 'maxbits' argument of str2int() does not work well,
  especially for =32 (which returns 0 because of overflow)
*/

var hex_array=[
 "0", "1", "2", "3", "4", "5", "6", "7",
 "8", "9", "A", "B", "C", "D", "E", "F"
];

var int_array={
"0":0, "1":1, "2":2,  "3":3, "4":4, "5":5, "6":6, "7":7,
"8":8, "9":9, "A":10, "B":11,"C":12,"D":13,"E":14,"F":15,
              "a":10, "b":11,"c":12,"d":13,"e":14,"f":15};

// input : an int or null, output : a string
// Beware : maxdigits, mindigits are off by -1
function int2hex(val, maxdigits, mindigits, opt_h) {

  if (val == null)
    return "u"; // null is used to mean "uninitialised"

  if (typeof val != "number") {
    add_message("orange", I8N("R")+I8N("wiffi"));
    return "??????";
  }

  var i=0;
  var b="h";

  if (typeof maxdigits != "number")
    maxdigits=7; // max number (-1) of char (default)

  if (typeof mindigits != "number")
    mindigits=0;

//    if ((typeof opt_h == 'number') && (val < 10))
  if (opt_h)
    b=""; // no trailing 'h'

  do {
    b = hex_array[val & 15] + b;
    val >>= 4;
    if ((val == 0) && (i>=mindigits))
      return b;
  } while (i++ < maxdigits);

  return b;
}

// wrapper, used in text inputs where spaces can creep in
function _str2int(t,add_message,maxbits) {
  return  str2int(t.replace(/(^\s*)|(\s*$)/g,""), add_message,maxbits);
}

// input : t is a string, returns : an int, or null if error
function str2int(t, add_message,maxbits) {
  if (typeof t != "string")
    return null;
  if (t.length < 1)
    return null;

  var i=0,
      a=0,
      b=0,
      c = t[t.length-1];

  if (((c == "h")||(c == "H")) && (t.length > 1)) {
// hex2int conversion :
    while (i < (t.length - 1)) {
      b=int_array[t[i]];
      if (typeof b != "number") {
        if (typeof add_message=="function")
          add_message("orange", I8N("R")+I8N("numforH"));
        return null;
      }
      a = (a << 4) + b;
      i++;
    }
  }

  else {
    if (((c == "b")||(c == "B")) && (t.length > 1)) {
// bin2int conversion :
      while (i < (t.length - 1)) {
        b=t.charCodeAt(i);
        if ((b & ~1)!=0x30) {
          if (typeof add_message=="function")
            add_message("orange", I8N("R")+I8N("numforB"));
          return null;
        }
        a += a + (b&1);
        i++;
      }
    }

    else {
// dec2int conversion :

      var signe=1; // check the sign
      if ((t.charAt(i)=="-")&&(t.length>1)) {
        signe=-1;
        i++;
      }

      while (i < (t.length)) {
        b = t.charCodeAt(i);
        if ((b < 0x30) || (b > 0x39)) {
          if (typeof add_message=="function")
            add_message("orange", I8N("R")+I8N("numforD"));
          return null;
        }
        a = (a * 10) + (b - 0x30);
        i++;
      }
      a*=signe; // this may cause some trouble close to the max values, be careful !
    }
  }

// security warning :
  if (typeof(maxbits)=="number") {
    var b=a;
    if (b<0)
      b=-b;  // might create off-by-ones around the powers of two... does 1-b solve this problem ?
    if ((b>=(1<<maxbits))
       && (typeof add_message=="function"))
      add_message("orange", I8N("W")+I8N("oob1")+maxbits+I8N("oob2"));
    a &= ((1<<maxbits) - 1);
  }

  return a; 
}

function toBin(n,l) { // or not toBin() ?
  var m="";
  while (l>0) {
    m= (n&1)+m;
    n>>=1;
    l--;
  }
  return m;
}
